@@ -5,8 +5,8 @@ var http = require('http');
 var fs = require('fs');
 var spawn = require('child_process').spawn;
 require('../common.js');
-var test = require('../../test/common.js');
-var pep = path.dirname(process.argv[1]) + '/_chunky_http_client.js';
+var test = require('../../test/common');
+var pep = `${path.dirname(process.argv[1])}/_chunky_http_client.js`;
 var PIPE = test.PIPE;
 
 try {
@@ -21,8 +21,11 @@ try {
 } catch (e) { /* ignore */ }
 
 server = http.createServer(function(req, res) {
-  res.writeHead(200, { 'content-type': 'text/plain',
-                       'content-length': '2' });
+  var headers = {
+    'content-type': 'text/plain',
+    'content-length': '2'
+  };
+  res.writeHead(200, headers);
   res.end('ok');
 });
 
@@ -51,4 +54,3 @@ try {
 } catch (e) {
   throw new Error('error: ' + e);
 }
-
