@@ -1,9 +1,9 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var net = require('net');
+const assert = require('assert');
+const net = require('net');
 
-var s = new net.Stream();
+const s = new net.Stream();
 
 // test that destroy called on a stream with a server only ever decrements the
 // server connection count once
@@ -12,17 +12,17 @@ s.server = new net.Server();
 s.server.connections = 10;
 s._server = s.server;
 
-assert.equal(10, s.server.connections);
+assert.strictEqual(10, s.server.connections);
 s.destroy();
-assert.equal(9, s.server.connections);
+assert.strictEqual(9, s.server.connections);
 s.destroy();
-assert.equal(9, s.server.connections);
+assert.strictEqual(9, s.server.connections);
 
-var SIZE = 2E6;
-var N = 10;
-var buf = Buffer.alloc(SIZE, 'a');
+const SIZE = 2E6;
+const N = 10;
+const buf = Buffer.alloc(SIZE, 'a');
 
-var server = net.createServer(function(socket) {
+const server = net.createServer(function(socket) {
   socket.setNoDelay();
 
   socket.on('error', function(err) {
@@ -31,15 +31,15 @@ var server = net.createServer(function(socket) {
     server.close();
   });
 
-  for (var i = 0; i < N; ++i) {
+  for (let i = 0; i < N; ++i) {
     socket.write(buf, function() { });
   }
   socket.end();
 
 }).listen(0, function() {
-  var conn = net.connect(this.address().port);
+  const conn = net.connect(this.address().port);
   conn.on('data', function(buf) {
-    conn.pause();
+    assert.strictEqual(conn, conn.pause());
     setTimeout(function() {
       conn.destroy();
     }, 20);
@@ -47,5 +47,5 @@ var server = net.createServer(function(socket) {
 });
 
 process.on('exit', function() {
-  assert.equal(server.connections, 0);
+  assert.strictEqual(server.connections, 0);
 });
