@@ -14,9 +14,9 @@ if (!process.argv[2]) {
   const spawn = require('child_process').spawn;
   const path = require('path');
 
-  const pipeNamePrefix = path.basename(__filename) + '.' + process.pid;
-  const stdinPipeName = '\\\\.\\pipe\\' + pipeNamePrefix + '.stdin';
-  const stdoutPipeName = '\\\\.\\pipe\\' + pipeNamePrefix + '.stdout';
+  const pipeNamePrefix = `${path.basename(__filename)}.${process.pid}`;
+  const stdinPipeName = `\\\\.\\pipe\\${pipeNamePrefix}.stdin`;
+  const stdoutPipeName = `\\\\.\\pipe\\${pipeNamePrefix}.stdout`;
 
   const stdinPipeServer = net.createServer(function(c) {
     c.on('end', common.mustCall(function() {
