@@ -1,13 +1,13 @@
 /* eslint-disable max-len */
 'use strict';
 require('../common');
-var assert = require('assert');
+const assert = require('assert');
 
-var url = require('url');
+const url = require('url');
 
 // URLs to parse, and expected data
 // { url : parsed }
-var parseTests = {
+const parseTests = {
   '//some_path': {
     href: '//some_path',
     pathname: '//some_path',
@@ -923,8 +923,8 @@ for (const u in parseTests) {
   expected = parseTests[u].href;
   actual = url.format(parseTests[u]);
 
-  assert.equal(actual, expected,
-               'format(' + u + ') == ' + u + '\nactual:' + actual);
+  assert.strictEqual(actual, expected,
+                     'format(' + u + ') == ' + u + '\nactual:' + actual);
 }
 
 function createWithNoPrototype(properties = []) {
@@ -944,7 +944,7 @@ function check(actual, expected) {
   });
 }
 
-var parseTestsWithQueryString = {
+const parseTestsWithQueryString = {
   '/foo/bar?baz=quux#frag': {
     href: '/foo/bar?baz=quux#frag',
     hash: '#frag',
@@ -1015,7 +1015,7 @@ for (const u in parseTestsWithQueryString) {
 
 // some extra formatting tests, just to verify
 // that it'll format slightly wonky content to a valid url.
-var formatTests = {
+const formatTests = {
   'http://example.com?': {
     href: 'http://example.com/?',
     protocol: 'http:',
@@ -1232,11 +1232,11 @@ for (const u in formatTests) {
   delete formatTests[u].href;
   const actual = url.format(u);
   const actualObj = url.format(formatTests[u]);
-  assert.equal(actual, expect,
-               'wonky format(' + u + ') == ' + expect +
+  assert.strictEqual(actual, expect,
+                     'wonky format(' + u + ') == ' + expect +
                '\nactual:' + actual);
-  assert.equal(actualObj, expect,
-               'wonky format(' + JSON.stringify(formatTests[u]) +
+  assert.strictEqual(actualObj, expect,
+                     'wonky format(' + JSON.stringify(formatTests[u]) +
                ') == ' + expect +
                '\nactual: ' + actualObj);
 }
@@ -1244,7 +1244,7 @@ for (const u in formatTests) {
 /*
  [from, path, expected]
 */
-var relativeTests = [
+const relativeTests = [
   ['/foo/bar/baz', 'quux', '/foo/bar/quux'],
   ['/foo/bar/baz', 'quux/asdf', '/foo/bar/quux/asdf'],
   ['/foo/bar/baz', 'quux/baz', '/foo/bar/quux/baz'],
@@ -1293,8 +1293,8 @@ var relativeTests = [
 relativeTests.forEach(function(relativeTest) {
   const a = url.resolve(relativeTest[0], relativeTest[1]);
   const e = relativeTest[2];
-  assert.equal(a, e,
-               'resolve(' + [relativeTest[0], relativeTest[1]] + ') == ' + e +
+  assert.strictEqual(a, e,
+                     'resolve(' + [relativeTest[0], relativeTest[1]] + ') == ' + e +
                '\n  actual=' + a);
 });
 
@@ -1323,7 +1323,7 @@ relativeTests.forEach(function(relativeTest) {
 //
 // Changes marked with @isaacs
 
-var bases = [
+const bases = [
   'http://a/b/c/d;p?q',
   'http://a/b/c/d;p?q=1/2',
   'http://a/b/c/d;p=1/2?q',
@@ -1332,7 +1332,7 @@ var bases = [
 ];
 
 //[to, from, result]
-var relativeTests2 = [
+const relativeTests2 = [
   // http://lists.w3.org/Archives/Public/uri/2004Feb/0114.html
   ['../c', 'foo:a/b', 'foo:c'],
   ['foo:.', 'foo:a', 'foo:'],
@@ -1625,8 +1625,8 @@ var relativeTests2 = [
 relativeTests2.forEach(function(relativeTest) {
   const a = url.resolve(relativeTest[1], relativeTest[0]);
   const e = relativeTest[2];
-  assert.equal(a, e,
-               'resolve(' + [relativeTest[1], relativeTest[0]] + ') == ' + e +
+  assert.strictEqual(a, e,
+                     'resolve(' + [relativeTest[1], relativeTest[0]] + ') == ' + e +
                '\n  actual=' + a);
 });
 
@@ -1635,8 +1635,8 @@ relativeTests2.forEach(function(relativeTest) {
 
 //format: [from, path, expected]
 relativeTests.forEach(function(relativeTest) {
-  var actual = url.resolveObject(url.parse(relativeTest[0]), relativeTest[1]);
-  var expected = url.parse(relativeTest[2]);
+  let actual = url.resolveObject(url.parse(relativeTest[0]), relativeTest[1]);
+  let expected = url.parse(relativeTest[2]);
 
 
   assert.deepStrictEqual(actual, expected);
@@ -1644,8 +1644,8 @@ relativeTests.forEach(function(relativeTest) {
   expected = relativeTest[2];
   actual = url.format(actual);
 
-  assert.equal(actual, expected,
-               'format(' + actual + ') == ' + expected + '\nactual:' + actual);
+  assert.strictEqual(actual, expected,
+                     'format(' + actual + ') == ' + expected + '\nactual:' + actual);
 });
 
 //format: [to, from, result]
@@ -1663,22 +1663,22 @@ if (relativeTests2[181][0] === './/g' &&
   relativeTests2.splice(181, 1);
 }
 relativeTests2.forEach(function(relativeTest) {
-  var actual = url.resolveObject(url.parse(relativeTest[1]), relativeTest[0]);
-  var expected = url.parse(relativeTest[2]);
+  let actual = url.resolveObject(url.parse(relativeTest[1]), relativeTest[0]);
+  let expected = url.parse(relativeTest[2]);
 
   assert.deepStrictEqual(actual, expected);
 
   expected = relativeTest[2];
   actual = url.format(actual);
 
-  assert.equal(actual, expected,
-               'format(' + relativeTest[1] + ') == ' + expected +
+  assert.strictEqual(actual, expected,
+                     'format(' + relativeTest[1] + ') == ' + expected +
                '\nactual:' + actual);
 });
 
 
 // https://github.com/nodejs/node/pull/1036
-var throws = [
+const throws = [
   undefined,
   null,
   true,
